// This file is part of ICU4X. For terms of use, please see the file
// called LICENSE at the top level of the ICU4X source tree
// (online at: https://github.com/unicode-org/icu4x/blob/main/LICENSE ).

#[allow(dead_code)]
pub const KEBAB_CASE_STRINGS: &[&str] = &[
    "ca",
    "ca-ÉS",
    "ca-ÉS-u-ca-buddhist",
    "ca-ÉS-valencia",
    "ca-ÉS-x-gbp",
    "ca-ÉS-x-gbp-short",
    "ca-ÉS-x-usd",
    "ca-ÉS-xyzabc",
    "ca-x-eur",
    "cat",
    "cat-bus",
    "cat-🚐",
    "pl-Latn-PL",
    "und",
    "und-fonipa",
    "und-u-ca-hebrew",
    "und-u-ca-japanese",
    "und-x-mxn",
    "zh",
];
