// Copyright 2021 Contributors to the Parsec project.
// SPDX-License-Identifier: Apache-2.0
mod algorithm_property_tests;
mod attest_info_test;
mod attest_tests;
mod buffers_tests;
mod capability_data_tests;
mod certify_info_tests;
mod clock_info_tests;
mod command_audit_info_tests;
mod creation_info_tests;
mod lists_tests;
mod nv_certify_info_tests;
mod pcr_tests;
mod quote_info_tests;
mod session_audit_info_tests;
mod tagged_pcr_select_tests;
mod tagged_property_tests;
mod tagged_tests;
mod time_attest_info_tests;
mod time_info_tests;
